import json
from pathlib import Path
from typing import Optional

import httpx
from tornado import web

from jupyter_server.auth.decorator import authorized
from jupyter_server.base.handlers import APIHandler


AUTH_RESOURCE = "terminals"


class TerminalAPIHandler(APIHandler):
    auth_resource = AUTH_RESOURCE


class TerminalRootHandler(TerminalAPIHandler):
    @web.authenticated
    @authorized
    async def get(self):
        async with httpx.AsyncClient() as client:
            response = await client.get(f"{self.settings['proxy_url']}/api/terminals")
        self.set_status(response.status_code)
        self.finish(response.text)

    @web.authenticated
    @authorized
    async def post(self):
        data = self.get_json_body() or {}
        async with httpx.AsyncClient() as client:
            response = await client.post(f"{self.settings['proxy_url']}/api/terminals", data=data)
        self.set_status(response.status_code)
        self.finish(response.json())


class TerminalHandler(TerminalAPIHandler):
    SUPPORTED_METHODS = ("GET", "DELETE")  # type:ignore[assignment]

    @web.authenticated
    @authorized
    async def get(self, name):
        url = f"{self.settings['proxy_url']}/api/terminals/{name}"
        async with httpx.AsyncClient() as client:
            response = await client.get(url)
        self.set_status(response.status_code)
        self.finish(response.json())

    @web.authenticated
    @authorized
    async def delete(self, name):
        url = f"{self.settings['proxy_url']}/api/terminals/{name}"
        async with httpx.AsyncClient() as client:
            response = await client.delete(url)
        self.set_status(response.status_code)
        self.finish()


default_handlers = [
    (r"/api/terminals", TerminalRootHandler),
    (r"/api/terminals/(\w+)", TerminalHandler),
]



from jupyter_server.extension.application import ExtensionApp
from traitlets import Unicode

from . import api_handlers, handlers


class TerminalsProxyExtensionApp(ExtensionApp):

    name = "jupyter_server_terminals_proxy"

    proxy_url = Unicode(
        help="The URL from where to proxy terminals."
    ).tag(config=True)

    def initialize_settings(self):
        self.settings.update(dict(terminals_available=True, proxy_url=self.proxy_url))

    def initialize_handlers(self):
        self.handlers.append(
            (
                r"/terminals/websocket/(\w+)",
                handlers.TermSocket,
            )
        )
        self.handlers.extend(api_handlers.default_handlers)
        self.serverapp.web_app.settings["terminals_available"] = self.settings[
            "terminals_available"
        ]
        #print(f"{self.serverapp.terminals_enabled=}")
        #self.serverapp.terminals_enabled = False




import asyncio

from tornado.websocket import WebSocketHandler
from tornado import web
from websockets import connect

from jupyter_server._tz import utcnow
from jupyter_server.auth.utils import warn_disabled_authorization
from jupyter_server.base.handlers import JupyterHandler

AUTH_RESOURCE = "terminals"


class TermSocket(WebSocketHandler, JupyterHandler):

    auth_resource = AUTH_RESOURCE

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.websocket = None

    def check_origin(self, origin):
        return True

    def get(self, *args, **kwargs):
        user = self.current_user

        if not user:
            raise web.HTTPError(403)

        # authorize the user.
        if not self.authorizer:
            # Warn if an authorizer is unavailable.
            warn_disabled_authorization()
        elif not self.authorizer.is_authorized(self, user, "execute", self.auth_resource):
            raise web.HTTPError(403)

        return super().get(*args, **kwargs)

    async def open(self, name):
        proxy_url = self.settings["proxy_url"]
        ws_url = "ws" + proxy_url[proxy_url.find(":"):]
        self.websocket = await connect(f"{ws_url}/terminals/websocket/{name}")
        asyncio.create_task(self.process_message())

    async def on_message(self, message):
        await self.websocket.send(message)
        self._update_activity()

    async def process_message(self):
        while True:
            try:
                message = await self.websocket.recv()
                self.write_message(message, binary=False)
                self._update_activity()
            except Exception:
                break

    def on_close(self):
        asyncio.create_task(self.websocket.close())
        self.websocket = None

    def _update_activity(self):
        self.application.settings["terminal_last_activity"] = utcnow()



from .app import TerminalsProxyExtensionApp


__version__ = "0.1.4"


def _jupyter_server_extension_points():  # pragma: no cover
    return [
        {
            "module": "jupyter_server_terminals_proxy.app",
            "app": TerminalsProxyExtensionApp,
        },
    ]
